#include "vars.inc"
errorlevel -302
	org		0x0000
	goto	main
	org		0x0004
#include "interrupt.inc"
#include "eeprom.inc"
#include "other.inc"

main
	
Start
	call	init
	call	ee_ewen		        ; output EWEN for enabling writes
	movlw	0xE0
	movwf	INTCON
	rrf		PORTB,W				;take PORTB state
	movwf	timer
	rrf		timer,W				;rotate twice (3..2 -> 1..0)
	andlw	0x03				;mask
	call	time_table			;take value
	movwf	timer				;
	goto	time_table_end		;save into variable
time_table
	addwf	PCL,f
	retlw	CYC1
	retlw	CYC2
	retlw	CYC3
	retlw	CYC4
time_table_end

main_loop					;jump into corresponding state
	movlw	0x00
	xorwf	state,W
	btfsc	STATUS,Z
	call	s0

	movlw	0x01
	xorwf	state,W
	btfsc	STATUS,Z
	call	s1

	movlw	0x02
	xorwf	state,W
	btfsc	STATUS,Z
	call	s2

 	goto	main_loop

;-----------------------------------------------------------------
;state 0, idle
s0
	btfss	KEY1						;pressed key 1?
	goto	s0_nk1
		nop
		bcf		KEY1					;yes, clear flag
		movlw	0x01					;move to state 1 (record)
		movwf	state
		clrf	keys					;clear counters
		clrf	int_count_secs
		return
s0_nk1
	btfss	KEY2						;pressed key 2?
	goto	s0_nk2
		nop
		bcf		KEY2					;yes, clear flag
		movlw	0x02					;and move to stae 2 (transmit)
		movwf	state
		clrf	keys					
		clrf	ee_addr_l				;clear memory pointers to start from 0
		clrf	ee_addr_h	
		return
s0_nk2									;no key pressed
	bcf		LEDG						;just sit and turn LEDR on
	bsf		LEDR
	bcf		RELAY						;relay off
	return

;-----------------------------------------------------------------
;state 1, record
s1
	btfss	KEY1						;key 1 pressed?
	goto	s1_nk1
		nop
		bcf		KEY1					;if so, then clar flag
s1_jmps0
		movlw	0x00					;and terminate this state
		movwf	state
		clrf	keys
		movlw	0xAA					;write 0xAAAA into EEPROM
		movwf	adres_l					;to distinguish end of write
		movwf	adres_h					;
		goto	s1_do_write				;and do it, ending with return, returning into state 0
s1_jmps0_nw								
		movlw	0x00					;return from state
		movwf	state	
		clrf	keys
		return
s1_nk1	
	bsf		LEDG						;if no key pressed, turn LEDG on
	bcf		LEDR
	bsf		RELAY						;and RELAY is on
	bsf		STATUS,RP1
	movf	CM2CON0,W					;take CMCON value
	bcf		STATUS,RP1
	andlw	0x40						;and if input voltage is lower than threshold
	btfss	STATUS,Z
	goto	s1_jmps0					;do the same as pressing key 1 does
	call	do_adc						;otherwise do AD conversion
	movf	timer,W						;
;	movlw	0x02
	subwf	int_count_secs,W			;check whether it is time to write
	btfsc	STATUS,C					;if so
	goto	s1_do_write					;do it
	nop									;or quit
	return

s1_do_write
	bsf		LEDR				;blink a LEDR
	call	ee_ewen		        ; output EWEN for enabling writes
	movf	cur_addr_h,W
	movwf	ee_addr_h
	movf	cur_addr_l,W
	movwf	ee_addr_l
	movf	adres_h,W
	movwf	ee_data
	call	ee_write
	incf	ee_addr_l
	movf	adres_l,W
	movwf	ee_data
	call	ee_write			;do EEPROM write
	movlw	0x02				;and check the EEPROM end
	addwf	cur_addr_l,f
	btfsc	STATUS,Z
	incf	cur_addr_h
	movlw	EE_RANGE_HIBYTE
	xorwf	cur_addr_h,W
	btfsc	STATUS,Z
	goto	s1_jmps0			;if reached, terminate this state

	clrf	int_count_secs		;otherwise clar counter and count again
	bcf		LEDR				;turn LEDR off (short blink)
	return

;-----------------------------------------------------------------
;state 2, transmit
s2
	btfss	KEY2				;check key 2
	goto	s2_nk1
		nop
		bcf		KEY2			;if pressed, stop and return to state 0
		movlw	0x00
		movwf	state
		clrf	keys
		return
s2_nk1
	bsf		LEDG				;turn both LEDs on
	bsf		LEDR
	bcf		RELAY				;and RELAY off

	call	ee_read				;read EE
	movwf	adres_h
	andlw	0xF0
	btfss	STATUS,Z	
	goto	s1_jmps0_nw
	incf	ee_addr_l
	call	ee_read
	movwf	adres_l
	call	Bin2DecFast			;convert into BCD
	movf	D3,W
	addlw	'0'
	call	output_tx			;and transmit one by one, with dec correction
	movf	D2,W
	addlw	'0'
	call	output_tx
	movf	D1,W
	addlw	'0'
	call	output_tx
	movf	D0,W
	addlw	'0'
	call	output_tx
	movlw	0x0D
	call	output_tx
	movlw	0x0A
	call	output_tx
	incf	ee_addr_l
	btfsc	STATUS,Z
	incf	ee_addr_h
	movlw	EE_RANGE_HIBYTE		;check for EE end
	xorwf	ee_addr_h,W
	btfsc	STATUS,Z
	goto	s1_jmps0_nw			;and jump out of this state, if needed
	return						;or return in this state

;---------------------------------------------------------------------
;transmit W, blocking function
output_tx		
	movwf	TXREG
	bsf		STATUS,RP0
output_tx_loop
	btfss	TXSTA,TRMT
	goto	output_tx_loop
	bcf		STATUS,RP0
	return

;---------------------------------------------------------------------
;binary to BCD, what else?
Bin2DecFast
	movf    adres_h,w
	iorlw   0xF0            ;w=H2-16
	movwf   D1              ;D1=H2-16
	addwf   D1,f            ;D1=H2*2-32
	addwf   D1,f            ;D1=H2*3-48
	movwf   D2              ;D2=H2-16
	addlw   -D'5'           ;w=H2-21
	addwf   D2,f            ;D2=H2*2-37 Done!
	addlw   D'41'           ;w=H2+20
	movwf   D0              ;D0=H2+20

	swapf   adres_l,w
	iorlw   0xF0            ;w=H1-16
	addwf   D1,f            ;D1=H2*3+H1-64
	addwf   D0,f            ;D0=H2+H1+4, C=1
	rlf     D0,f            ;D0=(H2+H1)*2+9, C=0
	comf    D0,f            ;D0=-(H2+H1)*2-10
	rlf     D0,f            ;D0=-(H2+H1)*4-20

	movf    adres_l,w
	andlw   0x0F            ;w=H0
	addwf   D0,f            ;D0=H0-(H2+H1)*4-20 Done!
	rlf     D1,f            ;C=0, D1=H2*6+H1*2-128 Done! 

	movlw   D'5'
	movwf   D3

	movlw   D'10'
mod0
	addwf   D0,f            ;D(X)=D(X)mod10
	decf    D1,f            ;D(X+1)=D(X+1)+D(X)div10
	skpc
	goto    mod0
mod1
	addwf   D1,f
	decf    D2,f
	skpc
	goto    mod1
mod2
	addwf   D2,f
	decf    D3,f
	skpc
	goto    mod2

    return




	end


